require '/data_importer/lib/importers/github_repo.rb'

class GithubAdvisoryImporter < GithubRepo
    # repo has years that begin with 2017 as first GHSA
    YEAR_RANGE = (2017..Date.today.year)

    def initialize
      super(repo_url='https://github.com/github/advisory-database.git', repo_path='/data_importer/data/github_advisories')
    end

    def advisory_paths
        advisory_path = "#{repo_path}/advisories"
        {
            :base_path => advisory_path,
            :github_reviewed_path => "#{advisory_path}/github-reviewed",
            :unreviewed_path => "#{advisory_path}/unreviewed"
        }
    end

    def list_jsons_for_year(year)
        json_wildcard = "*.json"
        github_reviewed_year_fp = "#{advisory_paths[:github_reviewed_path]}/#{year}/*/*"
        unreviewed_year_fp = "#{advisory_paths[:unreviewed_path]}/#{year}/*/*"
        
        github_reviewed_jsons_fp = Dir["#{github_reviewed_year_fp}/#{json_wildcard}"]
        unreviewed_jsons_fp = Dir["#{unreviewed_year_fp}/#{json_wildcard}"]

        {
            :github_reviewed_jsons => github_reviewed_jsons_fp,
            :unreviewed_jsons => unreviewed_jsons_fp
        }
      end
    
      def read_jsons_for_year(year)
        fp_hash = list_jsons_for_year(year)
        fns = fp_hash[:github_reviewed_jsons] + fp_hash[:unreviewed_jsons]
        jsons = fns.map do |fn|
            read_json(fn)
        end
        jsons.flatten
      end

      def attrs_from_item(json)
        attrs = {}
        attrs[:schema_version] = json['schema_version']
        attrs[:ghsa_id] = json['id']
        attrs[:modified] = json['modified']
        attrs[:published] = json['published']
        attrs[:aliases] = json['aliases']
        attrs[:summary] = json['summary']
        attrs[:details] = json['details']
        attrs[:severity] = json['severity']
        attrs[:affected] = json['affected']
        attrs[:references] = json['references']
        attrs[:database_specific] = json['database_specific']
        attrs
    end

    def bulk_insert(jsons)
        GithubAdvisory.bulk_insert do |worker|
          jsons.each do |json|
            attrs = attrs_from_item(json)
            worker.add(attrs)
          end
        end
    end

    def import
        pull_or_clone
        puts "Now importing GithubAdvisories."
        YEAR_RANGE.each do |year|
            puts "Importing advisory data from #{year}"
            jsons = read_jsons_for_year(year)
            bulk_insert(jsons)
        end
    end
end